--
-- Copyright (C) 2016 HE Yaowen
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--

local _, FlexBind = ...

-- Binding scheme example
FlexBind.schemes["example"] = {
    -- Whether load this binding scheme automatically
    -- optional, defaults to false
    autoload = false,

    -- Description of this binding setting
    -- optional, defaults to nil
    description = "descriptionText",

    -- Enable this binding set if the following conditions are matched.
    -- optional, defaults to nil
    -- Each condition is optional as well and treated as true if any item in parameters is matched.
    -- e.g. Instance = { "arena", "raid" } returns true if player in PvP arena OR a raid group.
    conditions = {
        -- Class names
        classes = { "className1", "className2" },

        -- Character names
        characters = { "characterName1", "characterName2" },

        -- Equipment set IDs
        equipmentSets = { 1, 2 },

        -- Instances
        -- @see http://wowprogramming.com/docs/api/IsInInstance
        instances = { "arena", "none" },

        -- Race names
        races = { "raceName1", "raceName2" },

        -- Realm names
        realms = { "realmName1", "realmName2" },

        -- Talent names
        talents = { "talentName1", "talentName2" },
    },

    -- Binding settings
    bindings = {
        -- Binding setting
        {
            -- Name of key which should be bound with the following action
            -- optional, defaults to nil
            key = "keyName",

            -- Binding action, can be a string, which means a COMMAND or a table means a button frame
            action = {
                -- Button frame, FlexBind will create a new action button if frame is missed
                -- optional, defaults to nil
                frame = ButtonFrame,

                -- Attributes of the button frame to be updated
                -- @see http://wowprogramming.com/docs/secure_template/SecureActionButtonTemplate
                attributes = {}
            },

            -- Description of this binding setting
            -- optional, defaults to nil
            description = "descriptionText",
        },

        -- Other binding settings
        -- ...
    }
}

FlexBind.schemes["example-priest-common"] = {
    autoload = true,
    description = "Example for priest common bindings",
    conditions = {
        classes = { "Priest" },
    },
    bindings = {
        -- Change movement bindings
        { key = "W", action = "MOVEFORWARD" },
        { key = "S", action = "MOVEBACKWARD" },
        { key = "A", action = "STRAFELEFT" },
        { key = "D", action = "STRAFERIGHT" },

        { key = "I", action = "OPENALLBAGS", description = "Open all bags if press 'I'" },

        {
            key = "1",
            action = { attributes = { type = "spell", spell = "Flash Heal" } },
            description = "Cast 'Flash Heal' to target if press '1'"
        },

        {
            key = "SHIFT-1",
            action = { attributes = { type = "spell", spell = "Flash Heal", unit = "player" } },
            description = "Cast 'Flash Heal' to player if press 'SHIFT+1' regardless what the target is"
        },

        {
            key = "2",
            action = { attributes = { type = "macro", macrotext = "/say I'm FlexBind :)" } },
            description = "Run standard macro if press '2'"
        },
        {
            action = {
                frame = PlayerFrame,
                attributes = {
                    -- Left button spell
                    type1 = "spell",
                    spell1 = "Flash Heal",

                    -- Right button spell
                    type2 = "spell",
                    spell2 = "Power Word: Shield",
                }
            },
            description = "Cast 'Flash Heal' to player if left click on player frame (avatar), and cast 'Power Word: Shield' if right click"
        },
    }
}

FlexBind.schemes["example-priest-shadow"] = {
    autoload = true,
    description = "Bindings just for shadow talent of priest",
    conditions = {
        classes = { "Priest" },
        talents = { "Shadow" }
    },
    bindings = {}
}
