--
-- Copyright (C) 2016 HE Yaowen
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--

local _, FlexBind = ...

local levelNames = { "ERROR", "WARNING", "INFO", "DEBUG" }

--- Prints a line in chat frame or console.
-- @param str string to print
--
function FlexBind_PrintLn(str)
    if FlexBind_Console_IsShown() then
        FlexBind_Console_PrintLn(str)
    else
        print(str)
    end
end

--- Prints log messages in game.
-- @param level log level
-- @param msg log message
--
local function LogMessage(level, msg)
    if level > FlexBind.config["logLevel"] then
        return
    end

    local str = string.format('[%s] %s', levelNames[level], msg)

    if not FlexBind_Console_IsShown() then
        str = "[FlexBind]" .. msg
    end

    FlexBind_PrintLn(str)
end


--- Logs error messages.
-- @param msg error message
--
function FlexBind_LogError(msg)
    LogMessage(1, "|cffff0000" .. msg .. "|r")
end

--- Logs warning messages.
-- @param msg warning message
--
function FlexBind_LogWarning(msg)
    LogMessage(2, "|cffff9900" .. msg .. "|r")
end

--- Logs information messages.
-- @param msg information message
--
function FlexBind_LogInfo(msg)
    LogMessage(3, "|cffffffff" .. msg .. "|r")
end

--- Logs debug messages.
-- @param msg debug message
--
function FlexBind_LogDebug(msg)
    LogMessage(4, "|cff00ffff" .. msg .. "|r")
end
